import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:http/http.dart' as http;
import 'package:intl/intl.dart';
import 'package:flutter_dotenv/flutter_dotenv.dart';
import 'package:textfield_tags/textfield_tags.dart';

import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import '../classes/alert.dart';
import '../classes/eventAdded.dart';

import '../variable/globals.dart' as globals;

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  Map<String, dynamic> events = {};
  String imagePath = "";
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      debugShowCheckedModeBanner: false,
      home: UpdateeventImage(events: events, imagePath: imagePath),
    );
  }
}

class UpdateeventImage extends StatefulWidget {
  const UpdateeventImage(
      {Key? key, required this.events, required this.imagePath})
      : super(key: key);
  final Map<String, dynamic> events;
  final String imagePath;

  @override
  _UpdateeventImageState createState() => _UpdateeventImageState();
}

class _UpdateeventImageState extends State<UpdateeventImage>
    with ShowErrorDialog, ShowEventDialog {
  TextEditingController inputName = TextEditingController();

  TextEditingController inputDate = TextEditingController();
  TextEditingController inputDesc = TextEditingController();

  TextEditingController inputGeo = TextEditingController();

  TextEditingController startDatepicker = TextEditingController();
  TextEditingController startTimepicker = TextEditingController();
  TextEditingController endDatepicker = TextEditingController();
  TextEditingController endTimepicker = TextEditingController();
  final _stringTagController = StringTagController();

  List<Map<String, dynamic>> suggestions = [];
  String geographicalZone = "";

  List<String> initialTags = [];

  final _stringOrgaController = StringTagController();
  List<String> initialOrga = [];

  onTapFunctionDatePicker(
      {required BuildContext context, required String position}) async {
    String date = "start_date";
    if (position == "end") {
      date = "end_date";
    }
    DateTime dateEvent;
    if (widget.events[date].toString().isEmpty) {
      dateEvent = DateTime.now();
    } else {
      dateEvent = DateTime.parse(widget.events[date]);
    }
    DateTime? pickedDate = await showDatePicker(
        context: context,
        firstDate: dateEvent,
        initialDate: dateEvent,
        lastDate: DateTime(2104));
    if (pickedDate == null) return;
    if (position == "start") {
      startDatepicker.text = DateFormat("dd-MM-yyyy").format(pickedDate);
    }
    if (position == "end") {
      endDatepicker.text = DateFormat("dd-MM-yyyy").format(pickedDate);
    }
  }

  onTapFunctionTimePicker(
      {required BuildContext context, required String position}) async {
    String date = "start_date";
    if (position == "end") {
      date = "end_date";
    }
    TimeOfDay timeEvent;
    if (widget.events[date].toString().isEmpty) {
      timeEvent = TimeOfDay.now();
    } else {
      timeEvent = TimeOfDay.fromDateTime(DateTime.parse(widget.events[date]));
    }
    TimeOfDay? pickedDate =
        await showTimePicker(context: context, initialTime: timeEvent);
    if (pickedDate == null) return;
    if (position == "start") {
      startTimepicker.text = pickedDate.format(context);
    }
    if (position == "end") {
      endTimepicker.text = pickedDate.format(context);
    }
  }

  convertNulltoEmptyString(var check) {
    if (check == null) {
      return "";
    }
    return check;
  }

  convertNulltoArray(List<String> check) {
    if (check == null) {
      return [];
    }
    return check;
  }

  String formatDate(String date) {
    var splitedDate = date.split("-");
    var day = splitedDate[0];
    var month = splitedDate[1];
    var year = splitedDate[2];

    return "${year}-${month}-${day}";
  }

  Future<void> _updateEvent(BuildContext context) async {
    var url = Uri.parse("${globals.api}/token");
    var name = inputName.text;
    var place = inputGeo.text;
    var description = inputDesc.text;
    List<String> tags = List<String>.from(_stringTagController.getTags as List);
    List<String> organizers =
        List<String>.from(_stringOrgaController.getTags as List);

    var startDateFormat = formatDate(startDatepicker.text);
    var endDateFormat = formatDate(endDatepicker.text);
    var startDate =
        "${startDateFormat}T${startTimepicker.text.replaceAll('-', ':')}";
    var endDate = "${endDateFormat}T${endTimepicker.text.replaceAll('-', ':')}";
    print("start date : ${startDate}");
    print("end date : ${endDate}");
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var accessToken = prefs.getString("access_token") ?? "";

    if (accessToken.isNotEmpty) {
      try {
        await dotenv.load();
        final mapboxAccessToken = dotenv.env['MAPBOX_ACCESS_TOKEN'] ?? '';
        print("place non encoded : ${place}");
        final url =
            'https://api.mapbox.com/geocoding/v5/mapbox.places/${place}.json?access_token=${mapboxAccessToken}&types=poi,address,place';
        var encoded = Uri.encodeFull(url);
        print("encoded : ${encoded}");
        final response = await http.get(Uri.parse(encoded));

        if (response.statusCode == 200) {
          final data = json.decode(response.body);
          print("data : ${data}");

          if (data['features'].isNotEmpty) {
            place = data['features'][0]['place_name'];
            final coordinates = data['features'][0]['geometry']['coordinates'];
            final longitude = coordinates[0]; // Longitude
            final latitude = coordinates[1]; // Latitude

            final params = {
              'expiration': '15552000',
              'key': dotenv.env["IMGBB_API_KEY"],
            };
            print("Post Img");
            final urlPost = Uri.parse('https://api.imgbb.com/1/upload')
                .replace(queryParameters: params);
            File image = File(widget.imagePath);
            Uint8List _bytes = await image.readAsBytes();
            String _base64String = base64.encode(_bytes);

            final req = http.MultipartRequest('POST', urlPost)
              ..fields['image'] = _base64String;

            final stream = await req.send();
            final res = await http.Response.fromStream(stream);

            final status = res.statusCode;
            print("code status imgbb ${status}");
            if (status == 200) {
              var body = json.decode(utf8.decode(res.bodyBytes));
              String imgUrl = body["data"]["url"];

              //String credentials = "${pseudo}:${password}";
              //Codec<String, String> stringToBase64 = utf8.fuse(base64);
              //String encoded = stringToBase64.encode(credentials);
              var urlPut = Uri.parse("${globals.api}/events");
              var responsePut = await http.put(urlPut,
                  headers: {
                    HttpHeaders.cookieHeader: 'access_token=${accessToken}',
                    HttpHeaders.acceptHeader:
                        'application/json, text/plain, */*',
                    HttpHeaders.contentTypeHeader: 'application/json'
                  },
                  body: jsonEncode({
                    'name': name,
                    'place': place,
                    'start_date': startDate,
                    'end_date': endDate,
                    'organizers': organizers,
                    'latitude': latitude,
                    'longitude': longitude,
                    'description': description,
                    "imgUrl": imgUrl,
                    "tags": tags
                  }));
              print(responsePut.statusCode);
              if ((responsePut.statusCode == 200) ||
                  (responsePut.statusCode == 201)) {
                showEventDialog(context, "Evenement ${name} ajoute");
              } else {
                var text = "";
                switch (responsePut.statusCode) {
                  case 400:
                    {
                      text = "Requête mal construite";
                    }
                    break;
                  case 406:
                    {
                      text = "Mot de passe incorrect";
                    }
                    break;
                  case 404:
                    {
                      text = "Utilisateur inconnu";
                    }
                    break;
                  case 403:
                    {
                      text = "Utilisateur desactive";
                    }
                    break;
                  case 410:
                    {
                      text = "Token invalide";
                    }
                    break;
                  case 500:
                    {
                      text = "Probleme interne du serveur";
                    }
                    break;
                  default:
                    {
                      text = "Probleme d'authentification inconnu";
                    }
                    break;
                }
                showErrorDialog(context, text);
              }
            } else {
              print("imgbb error : ${status}");
            }
          } else {
            showErrorDialog(context, "Aucune donnée geographique");
          }
        } else {
          showErrorDialog(context, "Mapbox non accessible");
        }
      } catch (e) {
        showErrorDialog(context, "${e}");
      }
    } else {
      showErrorDialog(context, "Champ vide");
    }
  }

  void start() async {
    print("events : ${widget.events}");
    inputName.text = convertNulltoEmptyString(widget.events["name"]);
    inputGeo.text = convertNulltoEmptyString(widget.events["place"]);
    inputDesc.text = convertNulltoEmptyString(widget.events["description"]);
    if (widget.events["start_date"].toString().isNotEmpty) {
      DateTime pickedStartDate =
          DateTime.parse(convertNulltoEmptyString(widget.events["start_date"]));
      startDatepicker.text = DateFormat("dd-MM-yyyy").format(pickedStartDate);
      startTimepicker.text = DateFormat("HH-mm").format(pickedStartDate);
    }
    if (widget.events["end_date"].toString().isNotEmpty) {
      DateTime pickedEndDate =
          DateTime.parse(convertNulltoEmptyString(widget.events["end_date"]));
      endDatepicker.text = DateFormat("dd-MM-yyyy").format(pickedEndDate);
      endTimepicker.text = DateFormat("HH-mm").format(pickedEndDate);
    }
    initialTags = List<String>.from(widget.events['tags'] as List);
    initialOrga = List<String>.from(widget.events['organizers'] as List);
  }

  @override
  void initState() {
    start();
    super.initState();
  }

  final _formKey = GlobalKey<FormState>();
  String? _validateField(String? value) {
    return value!.isEmpty ? 'Champ requis' : null;
  }

  Future<void> searchSuggestions(String input) async {
    await dotenv.load(fileName: ".env"); // Load .env file

    final mapboxAccessToken = dotenv.env['MAPBOX_ACCESS_TOKEN'] ?? '';
    final url =
        'https://api.mapbox.com/geocoding/v5/mapbox.places/${input}.json?access_token=${mapboxAccessToken}&types=poi,address,place';
    var encoded = Uri.encodeFull(url);
    final response = await http.get(Uri.parse(encoded));
    print("response code suggesttion : ${response.statusCode}");

    if (response.statusCode == 200) {
      final data = json.decode(response.body);
      print("data suggestion : ${data}");
      setState(() {
        suggestions = (data['features'] as List)
            .map((feature) => {
                  'place_name': feature['place_name'],
                  'text': feature['text'],
                  'geometry': feature[
                      'geometry'], // Include geometry for latitude/longitude
                })
            .toList();
      });
    } else {
      throw Exception('Failed to load suggestions');
    }
  }

  Padding _buildGeographicalZoneSearchField() {
    return Padding(
      padding:
          const EdgeInsets.only(left: 15.0, right: 15.0, top: 15, bottom: 0),
      child: Column(
        children: [
          TextField(
            controller: inputGeo,
            decoration: InputDecoration(
              labelText: 'Lieu',
              border: OutlineInputBorder(),
              suffixIcon: IconButton(
                icon: const Icon(Icons.clear),
                onPressed: () {
                  setState(() {
                    inputGeo.clear(); // Clear the text field
                    geographicalZone = ''; // Reset the geographical zone state
                    suggestions.clear(); // Optionally clear suggestions
                  });
                },
              ),
            ),
            onChanged: (value) {
              setState(() {
                geographicalZone = value;
                searchSuggestions(value);
              });
            },
          ),
          if (suggestions.isNotEmpty)
            Container(
              height: 200,
              decoration: BoxDecoration(
                border: Border.all(color: Colors.blue),
                borderRadius: BorderRadius.circular(8),
              ),
              child: ListView.builder(
                shrinkWrap: true,
                itemCount: suggestions.length,
                itemBuilder: (context, index) {
                  return ListTile(
                    title: Text(suggestions[index]['text']),
                    subtitle: Text(suggestions[index]['place_name']),
                    onTap: () async {
                      print("suggestion tapped : ${suggestions[index]}");
                      final latitude =
                          suggestions[index]['geometry']['coordinates'][1];
                      final longitude =
                          suggestions[index]['geometry']['coordinates'][0];

                      setState(() {
                        geographicalZone = suggestions[index]['text'];
                        inputGeo.text = geographicalZone;
                        suggestions.clear();
                      });
                    },
                  );
                },
              ),
            ),
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          title: Text("Add or Update a event"),
          backgroundColor: Colors.blue,
          foregroundColor: Colors.white,
        ),
        body: Form(
          key: _formKey,
          child: SingleChildScrollView(
            child: Column(
              children: <Widget>[
                Padding(
                  padding: const EdgeInsets.only(top: 60.0),
                  child: Center(
                    child: Container(
                        width: 200,
                        height: 150,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(100.0)),
                        child: Image.file(File(widget.imagePath))),
                  ),
                ),
                Padding(
                  //padding: const EdgeInsets.only(left:15.0,right: 15.0,top:0,bottom: 0),
                  padding: EdgeInsets.symmetric(horizontal: 15),
                  child: TextFormField(
                    controller: inputName,
                    validator: (value) => _validateField(value),
                    decoration: InputDecoration(
                        border: OutlineInputBorder(),
                        labelText: 'Nom',
                        hintText: 'Modifier le nom de l\'évènement'),
                  ),
                ),
                _buildGeographicalZoneSearchField(),
                Padding(
                  padding: const EdgeInsets.only(
                      left: 15.0, right: 15.0, top: 15, bottom: 0),
                  //padding: EdgeInsets.symmetric(horizontal: 15),
                  child: TextFormField(
                      controller: startDatepicker,
                      readOnly: true,
                      validator: (value) => _validateField(value),
                      decoration: InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Date de debut',
                          hintText: 'Cliquez ici pour selectionner une date'),
                      onTap: () => onTapFunctionDatePicker(
                          context: context, position: "start")),
                ),
                Padding(
                  padding: const EdgeInsets.only(
                      left: 15.0, right: 15.0, top: 15, bottom: 0),
                  //padding: EdgeInsets.symmetric(horizontal: 15),
                  child: TextFormField(
                      controller: startTimepicker,
                      readOnly: true,
                      validator: (value) => _validateField(value),
                      decoration: InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Heure de debut',
                          hintText: 'Cliquez ici pour selectionner une heure'),
                      onTap: () => onTapFunctionTimePicker(
                          context: context, position: "start")),
                ),
                Padding(
                  padding: const EdgeInsets.only(
                      left: 15.0, right: 15.0, top: 15, bottom: 0),
                  //padding: EdgeInsets.symmetric(horizontal: 15),
                  child: TextFormField(
                      controller: endDatepicker,
                      readOnly: true,
                      validator: (value) => _validateField(value),
                      decoration: InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Date de fin',
                          hintText: 'Cliquez ici pour selectionner une date'),
                      onTap: () => onTapFunctionDatePicker(
                          context: context, position: "end")),
                ),
                Padding(
                  padding: const EdgeInsets.only(
                      left: 15.0, right: 15.0, top: 15, bottom: 0),
                  //padding: EdgeInsets.symmetric(horizontal: 15),
                  child: TextFormField(
                      controller: endTimepicker,
                      readOnly: true,
                      validator: (value) => _validateField(value),
                      decoration: InputDecoration(
                          border: OutlineInputBorder(),
                          labelText: 'Heure de fin',
                          hintText: 'Cliquez ici pour selectionner une heure'),
                      onTap: () => onTapFunctionTimePicker(
                          context: context, position: "end")),
                ),
                TextFieldTags<String>(
                    textfieldTagsController: _stringTagController,
                    initialTags: initialTags,
                    textSeparators: const [' ', ','],
                    validator: (String tag) {
                      if (_stringTagController.getTags!.contains(tag)) {
                        return 'Tu as deja rentre ce tag';
                      }
                      return null;
                    },
                    inputFieldBuilder: (context, inputFieldValues) {
                      return Padding(
                        padding: const EdgeInsets.only(
                            left: 15.0, right: 15.0, top: 15, bottom: 0),
                        child: TextField(
                          controller: inputFieldValues.textEditingController,
                          focusNode: inputFieldValues.focusNode,
                          onChanged: inputFieldValues.onTagChanged,
                          onSubmitted: inputFieldValues.onTagSubmitted,
                          decoration: InputDecoration(
                            border: OutlineInputBorder(),
                            labelText: 'Tags',
                            hintText: inputFieldValues.tags.isNotEmpty
                                ? ''
                                : "Enter tag...",
                            errorText: inputFieldValues.error,
                            prefixIcon: inputFieldValues.tags.isNotEmpty
                                ? SingleChildScrollView(
                                    controller:
                                        inputFieldValues.tagScrollController,
                                    scrollDirection: Axis.vertical,
                                    child: Padding(
                                      padding: const EdgeInsets.only(
                                        top: 8,
                                        bottom: 8,
                                        left: 8,
                                      ),
                                      child: Wrap(
                                          runSpacing: 4.0,
                                          spacing: 4.0,
                                          children: inputFieldValues.tags
                                              .map((String tag) {
                                            return Container(
                                              decoration: const BoxDecoration(
                                                borderRadius: BorderRadius.all(
                                                  Radius.circular(20.0),
                                                ),
                                                color: Colors.blue,
                                              ),
                                              margin:
                                                  const EdgeInsets.symmetric(
                                                      horizontal: 5.0),
                                              padding:
                                                  const EdgeInsets.symmetric(
                                                      horizontal: 10.0,
                                                      vertical: 5.0),
                                              child: Row(
                                                mainAxisAlignment:
                                                    MainAxisAlignment.start,
                                                mainAxisSize: MainAxisSize.min,
                                                children: [
                                                  InkWell(
                                                    child: Text(
                                                      '$tag',
                                                      style: const TextStyle(
                                                          color: Colors.white),
                                                    ),
                                                    onTap: () {
                                                      //print("$tag selected");
                                                    },
                                                  ),
                                                  const SizedBox(width: 4.0),
                                                  InkWell(
                                                    child: const Icon(
                                                      Icons.cancel,
                                                      size: 14.0,
                                                      color: Color.fromARGB(
                                                          255, 233, 233, 233),
                                                    ),
                                                    onTap: () {
                                                      inputFieldValues
                                                          .onTagRemoved(tag);
                                                    },
                                                  )
                                                ],
                                              ),
                                            );
                                          }).toList()),
                                    ),
                                  )
                                : null,
                          ),
                        ),
                      );
                    }),
                TextFieldTags<String>(
                    textfieldTagsController: _stringOrgaController,
                    initialTags: initialOrga,
                    textSeparators: const [','],
                    validator: (String tag) {
                      if (_stringOrgaController.getTags!.contains(tag)) {
                        return 'Cet organisateur est déjà rentré';
                      }
                      return null;
                    },
                    inputFieldBuilder: (context, inputFieldValues) {
                      return Padding(
                        padding: const EdgeInsets.only(
                            left: 15.0, right: 15.0, top: 15, bottom: 0),
                        child: TextField(
                          controller: inputFieldValues.textEditingController,
                          focusNode: inputFieldValues.focusNode,
                          onChanged: inputFieldValues.onTagChanged,
                          onSubmitted: inputFieldValues.onTagSubmitted,
                          decoration: InputDecoration(
                            border: OutlineInputBorder(),
                            labelText: 'Organisateurs',
                            hintText: inputFieldValues.tags.isNotEmpty
                                ? ''
                                : "Enter un organisateur...",
                            errorText: inputFieldValues.error,
                            prefixIcon: inputFieldValues.tags.isNotEmpty
                                ? SingleChildScrollView(
                                    controller:
                                        inputFieldValues.tagScrollController,
                                    scrollDirection: Axis.vertical,
                                    child: Padding(
                                      padding: const EdgeInsets.only(
                                        top: 8,
                                        bottom: 8,
                                        left: 8,
                                      ),
                                      child: Wrap(
                                          runSpacing: 4.0,
                                          spacing: 4.0,
                                          children: inputFieldValues.tags
                                              .map((String tag) {
                                            return Container(
                                              decoration: const BoxDecoration(
                                                borderRadius: BorderRadius.all(
                                                  Radius.circular(20.0),
                                                ),
                                                color: Colors.blue,
                                              ),
                                              margin:
                                                  const EdgeInsets.symmetric(
                                                      horizontal: 5.0),
                                              padding:
                                                  const EdgeInsets.symmetric(
                                                      horizontal: 10.0,
                                                      vertical: 5.0),
                                              child: Row(
                                                mainAxisAlignment:
                                                    MainAxisAlignment.start,
                                                mainAxisSize: MainAxisSize.min,
                                                children: [
                                                  InkWell(
                                                    child: Text(
                                                      '$tag',
                                                      style: const TextStyle(
                                                          color: Colors.white),
                                                    ),
                                                    onTap: () {
                                                      //print("$tag selected");
                                                    },
                                                  ),
                                                  const SizedBox(width: 4.0),
                                                  InkWell(
                                                    child: const Icon(
                                                      Icons.cancel,
                                                      size: 14.0,
                                                      color: Color.fromARGB(
                                                          255, 233, 233, 233),
                                                    ),
                                                    onTap: () {
                                                      inputFieldValues
                                                          .onTagRemoved(tag);
                                                    },
                                                  )
                                                ],
                                              ),
                                            );
                                          }).toList()),
                                    ),
                                  )
                                : null,
                          ),
                        ),
                      );
                    }),
                Padding(
                  padding: const EdgeInsets.only(
                      left: 15.0, right: 15.0, top: 15, bottom: 0),
                  //padding: EdgeInsets.symmetric(horizontal: 15),
                  child: TextField(
                    controller: inputDesc,
                    keyboardType: TextInputType.multiline,
                    maxLines: 10,
                    decoration: InputDecoration(
                        border: OutlineInputBorder(),
                        labelText: 'Description',
                        hintText: 'Décrire l\'evènement'),
                  ),
                ),
                SizedBox(
                  height: 30,
                ),
                Container(
                  height: 50,
                  width: 250,
                  decoration: BoxDecoration(
                      color: Colors.blue,
                      borderRadius: BorderRadius.circular(20)),
                  child: TextButton(
                    onPressed: () {
                      if (_formKey.currentState!.validate()) {
                        _updateEvent(context);
                      }
                    },
                    child: Text(
                      'Ajouter',
                      style: TextStyle(color: Colors.white, fontSize: 25),
                    ),
                  ),
                )
              ],
            ),
          ),
        ));
  }
}
